// Code generated by github.com/atombender/go-jsonschema, DO NOT EDIT.

package test

type Schema struct {
	// Prop corresponds to the JSON schema field "prop".
	Prop SchemaBase `json:"prop,omitempty,omitzero" yaml:"prop,omitempty" mapstructure:"prop,omitempty"`
}

type SchemaBase []int
