// Code generated by github.com/atombender/go-jsonschema, DO NOT EDIT.

package test

type Map struct {
	// MyMap corresponds to the JSON schema field "myMap".
	MyMap MapMyMap `json:"myMap,omitempty,omitzero" yaml:"myMap,omitempty" mapstructure:"myMap,omitempty"`
}

type MapMyMap map[string]float64
