// Code generated by github.com/atombender/go-jsonschema, DO NOT EDIT.

package test

type BoolThing *bool

type FloatThing *float64

type IntegerThing *int

type NullableType struct {
	// MyInlineStringValue corresponds to the JSON schema field "MyInlineStringValue".
	MyInlineStringValue NullableTypeMyInlineStringValue `json:"MyInlineStringValue,omitempty,omitzero" yaml:"MyInlineStringValue,omitempty" mapstructure:"MyInlineStringValue,omitempty"`

	// MyStringValue corresponds to the JSON schema field "MyStringValue".
	MyStringValue StringThing `json:"MyStringValue,omitempty,omitzero" yaml:"MyStringValue,omitempty" mapstructure:"MyStringValue,omitempty"`
}

type NullableTypeMyInlineStringValue *string

type StringThing *string
