// Code generated by github.com/atombender/go-jsonschema, DO NOT EDIT.

package other

type Thing struct {
	// S corresponds to the JSON schema field "s".
	S *string `json:"s,omitempty,omitzero" yaml:"s,omitempty" mapstructure:"s,omitempty"`
}
