// Code generated by github.com/atombender/go-jsonschema, DO NOT EDIT.

package test

type Case struct {
	// CapitalCamelField corresponds to the JSON schema field "CapitalCamelField".
	CapitalCamelField *string `json:"CapitalCamelField,omitempty,omitzero" yaml:"CapitalCamelField,omitempty" mapstructure:"CapitalCamelField,omitempty"`

	// UPPERCASEFIELD corresponds to the JSON schema field "UPPERCASEFIELD".
	UPPERCASEFIELD *string `json:"UPPERCASEFIELD,omitempty,omitzero" yaml:"UPPERCASEFIELD,omitempty" mapstructure:"UPPERCASEFIELD,omitempty"`

	// CamelCase corresponds to the JSON schema field "camelCase".
	CamelCase *string `json:"camelCase,omitempty,omitzero" yaml:"camelCase,omitempty" mapstructure:"camelCase,omitempty"`

	// Lowercase corresponds to the JSON schema field "lowercase".
	Lowercase *string `json:"lowercase,omitempty,omitzero" yaml:"lowercase,omitempty" mapstructure:"lowercase,omitempty"`

	// SnakeMixedCase corresponds to the JSON schema field "snake_Mixed_Case".
	SnakeMixedCase *string `json:"snake_Mixed_Case,omitempty,omitzero" yaml:"snake_Mixed_Case,omitempty" mapstructure:"snake_Mixed_Case,omitempty"`

	// SnakeCase corresponds to the JSON schema field "snake_case".
	SnakeCase *string `json:"snake_case,omitempty,omitzero" yaml:"snake_case,omitempty" mapstructure:"snake_case,omitempty"`
}
