// Code generated by github.com/atombender/go-jsonschema, DO NOT EDIT.

package test

import "encoding/json"
import yaml "gopkg.in/yaml.v3"

type Thing struct {
	// Values corresponds to the JSON schema field "values".
	Values []Value `json:"values,omitempty,omitzero" yaml:"values,omitempty" mapstructure:"values,omitempty"`
}

// UnmarshalJSON implements json.Unmarshaler.
func (j *Thing) UnmarshalJSON(value []byte) error {
	type Plain Thing
	var plain Plain
	if err := json.Unmarshal(value, &plain); err != nil {
		return err
	}
	*j = Thing(plain)
	return nil
}

// UnmarshalYAML implements yaml.Unmarshaler.
func (j *Thing) UnmarshalYAML(value *yaml.Node) error {
	type Plain Thing
	var plain Plain
	if err := value.Decode(&plain); err != nil {
		return err
	}
	*j = Thing(plain)
	return nil
}

type Value float64

type AllOf = Thing
