// Code generated by github.com/atombender/go-jsonschema, DO NOT EDIT.

package test

type ObjectEmpty struct {
	// Foo corresponds to the JSON schema field "foo".
	Foo ObjectEmptyFoo `json:"foo,omitempty,omitzero" yaml:"foo,omitempty" mapstructure:"foo,omitempty"`
}

type ObjectEmptyFoo map[string]interface{}
