// Code generated by github.com/atombender/go-jsonschema, DO NOT EDIT.

package test

type Bar struct {
	// RefToFoo corresponds to the JSON schema field "refToFoo".
	RefToFoo *Foo `json:"refToFoo,omitempty,omitzero" yaml:"refToFoo,omitempty" mapstructure:"refToFoo,omitempty"`
}

type Cyclic struct {
	// A corresponds to the JSON schema field "a".
	A *Foo `json:"a,omitempty,omitzero" yaml:"a,omitempty" mapstructure:"a,omitempty"`
}

type Foo struct {
	// RefToBar corresponds to the JSON schema field "refToBar".
	RefToBar *Bar `json:"refToBar,omitempty,omitzero" yaml:"refToBar,omitempty" mapstructure:"refToBar,omitempty"`
}
