// Code generated by github.com/atombender/go-jsonschema, DO NOT EDIT.

package test

import "time"

type DateTimeNullable struct {
	// MyObject corresponds to the JSON schema field "myObject".
	MyObject *DateTimeNullableMyObject `json:"myObject,omitempty,omitzero" yaml:"myObject,omitempty" mapstructure:"myObject,omitempty"`
}

type DateTimeNullableMyObject struct {
	// MyNullableDateTime corresponds to the JSON schema field "myNullableDateTime".
	MyNullableDateTime *time.Time `json:"myNullableDateTime,omitempty,omitzero" yaml:"myNullableDateTime,omitempty" mapstructure:"myNullableDateTime,omitempty"`
}
