// Code generated by github.com/atombender/go-jsonschema, DO NOT EDIT.

package test

type Nillability struct {
	// Name corresponds to the JSON schema field "name".
	Name map[bool]string `json:"name,omitempty,omitzero" yaml:"name,omitempty" mapstructure:"name,omitempty"`
}
