// Code generated by github.com/atombender/go-jsonschema, DO NOT EDIT.

package test

type RefExternalFile struct {
	// MyExternalThing corresponds to the JSON schema field "myExternalThing".
	MyExternalThing *YamlStructNameFromFile `json:"myExternalThing,omitempty,omitzero" yaml:"myExternalThing,omitempty" mapstructure:"myExternalThing,omitempty"`

	// SomeOtherExternalThing corresponds to the JSON schema field
	// "someOtherExternalThing".
	SomeOtherExternalThing *YamlStructNameFromFile `json:"someOtherExternalThing,omitempty,omitzero" yaml:"someOtherExternalThing,omitempty" mapstructure:"someOtherExternalThing,omitempty"`
}

type YamlStructNameFromFile struct {
	// Foo corresponds to the JSON schema field "foo".
	Foo *string `json:"foo,omitempty,omitzero" yaml:"foo,omitempty" mapstructure:"foo,omitempty"`
}
