// Code generated by github.com/atombender/go-jsonschema, DO NOT EDIT.

package test

import "time"

type AnyOfNullableDateTime struct {
	// Event time as an RFC 3339 timestamp or Unix epoch seconds
	EventTime interface{} `json:"eventTime,omitempty,omitzero" yaml:"eventTime,omitempty" mapstructure:"eventTime,omitempty"`

	// ValidationTime corresponds to the JSON schema field "validationTime".
	ValidationTime *time.Time `json:"validationTime,omitempty,omitzero" yaml:"validationTime,omitempty" mapstructure:"validationTime,omitempty"`
}

type AnyOfNullableDateTimeEventTime_0 *time.Time

type AnyOfNullableDateTimeEventTime_1 *int
