// Code generated by github.com/atombender/go-jsonschema, DO NOT EDIT.

package test

type Issue51 struct {
	// Name corresponds to the JSON schema field "name".
	Name *string `json:"name,omitempty,omitzero" yaml:"name,omitempty" mapstructure:"name,omitempty"`

	AdditionalProperties interface{} `mapstructure:",remain"`
}
