// Code generated by github.com/atombender/go-jsonschema, DO NOT EDIT.

package test

type TypeMultiple struct {
	// All corresponds to the JSON schema field "all".
	All interface{} `json:"all,omitempty,omitzero" yaml:"all,omitempty" mapstructure:"all,omitempty"`

	// AllPrimitives corresponds to the JSON schema field "allPrimitives".
	AllPrimitives interface{} `json:"allPrimitives,omitempty,omitzero" yaml:"allPrimitives,omitempty" mapstructure:"allPrimitives,omitempty"`

	// ArrayOfAll corresponds to the JSON schema field "arrayOfAll".
	ArrayOfAll []interface{} `json:"arrayOfAll,omitempty,omitzero" yaml:"arrayOfAll,omitempty" mapstructure:"arrayOfAll,omitempty"`

	// ArrayOfAllPrimitives corresponds to the JSON schema field
	// "arrayOfAllPrimitives".
	ArrayOfAllPrimitives []interface{} `json:"arrayOfAllPrimitives,omitempty,omitzero" yaml:"arrayOfAllPrimitives,omitempty" mapstructure:"arrayOfAllPrimitives,omitempty"`

	// OnlyTwoOptions corresponds to the JSON schema field "onlyTwoOptions".
	OnlyTwoOptions interface{} `json:"onlyTwoOptions,omitempty,omitzero" yaml:"onlyTwoOptions,omitempty" mapstructure:"onlyTwoOptions,omitempty"`
}
