// Code generated by github.com/atombender/go-jsonschema, DO NOT EDIT.

package test

// A simple schema.
type Description struct {
	// MyDescriptionlessField corresponds to the JSON schema field
	// "myDescriptionlessField".
	MyDescriptionlessField *string `json:"myDescriptionlessField,omitempty,omitzero" yaml:"myDescriptionlessField,omitempty" mapstructure:"myDescriptionlessField,omitempty"`

	// A string field.
	MyField *string `json:"myField,omitempty,omitzero" yaml:"myField,omitempty" mapstructure:"myField,omitempty"`
}
