// Code generated by github.com/atombender/go-jsonschema, DO NOT EDIT.

package test

type RefToMap struct {
	// MyThing corresponds to the JSON schema field "myThing".
	MyThing map[string]float64 `json:"myThing,omitempty,omitzero" yaml:"myThing,omitempty" mapstructure:"myThing,omitempty"`
}
