// Code generated by github.com/atombender/go-jsonschema, DO NOT EDIT.

package other

type Thing struct {
	// Value corresponds to the JSON schema field "value".
	Value *Value `json:"value,omitempty,omitzero" yaml:"value,omitempty" mapstructure:"value,omitempty"`
}

type Value float64
