// Code generated by github.com/atombender/go-jsonschema, DO NOT EDIT.

package test

type Tags struct {
	// Html corresponds to the JSON schema field "html".
	Html *string `yaml:"html,omitempty"`

	// Id corresponds to the JSON schema field "id".
	Id *string `yaml:"id,omitempty"`

	// Url corresponds to the JSON schema field "url".
	Url *string `yaml:"url,omitempty"`
}
