// Code generated by github.com/atombender/go-jsonschema, DO NOT EDIT.

package test

type ObjectNested struct {
	// MyObject corresponds to the JSON schema field "myObject".
	MyObject *ObjectNestedMyObject `json:"myObject,omitempty,omitzero" yaml:"myObject,omitempty" mapstructure:"myObject,omitempty"`
}

type ObjectNestedMyObject struct {
	// MyObject corresponds to the JSON schema field "myObject".
	MyObject *ObjectNestedMyObjectMyObject `json:"myObject,omitempty,omitzero" yaml:"myObject,omitempty" mapstructure:"myObject,omitempty"`
}

type ObjectNestedMyObjectMyObject struct {
	// MyString corresponds to the JSON schema field "myString".
	MyString *string `json:"myString,omitempty,omitzero" yaml:"myString,omitempty" mapstructure:"myString,omitempty"`
}
