// Code generated by github.com/atombender/go-jsonschema, DO NOT EDIT.

package schema

import other "github.com/atombender/go-jsonschema/tests/data/crossPackageAllOf/other"

type Name string

type Schema struct {
	// DefInOtherSchema corresponds to the JSON schema field "defInOtherSchema".
	DefInOtherSchema *SchemaDefInOtherSchema `json:"defInOtherSchema,omitempty,omitzero" yaml:"defInOtherSchema,omitempty" mapstructure:"defInOtherSchema,omitempty"`

	// DefInSameSchema corresponds to the JSON schema field "defInSameSchema".
	DefInSameSchema *SchemaDefInSameSchema `json:"defInSameSchema,omitempty,omitzero" yaml:"defInSameSchema,omitempty" mapstructure:"defInSameSchema,omitempty"`
}

type SchemaDefInOtherSchema struct {
	// Value corresponds to the JSON schema field "value".
	Value *other.Value `json:"value,omitempty,omitzero" yaml:"value,omitempty" mapstructure:"value,omitempty"`
}

type SchemaDefInSameSchema struct {
	// Name corresponds to the JSON schema field "name".
	Name *Name `json:"name,omitempty,omitzero" yaml:"name,omitempty" mapstructure:"name,omitempty"`
}

type Thing struct {
	// Name corresponds to the JSON schema field "name".
	Name *Name `json:"name,omitempty,omitzero" yaml:"name,omitempty" mapstructure:"name,omitempty"`
}
