// Code generated by github.com/atombender/go-jsonschema, DO NOT EDIT.

package schema

import other "github.com/atombender/go-jsonschema/tests/data/crossPackage/other"

type Schema struct {
	// DefInOtherSchema corresponds to the JSON schema field "defInOtherSchema".
	DefInOtherSchema *other.Thing `json:"defInOtherSchema,omitempty,omitzero" yaml:"defInOtherSchema,omitempty" mapstructure:"defInOtherSchema,omitempty"`

	// DefInSameSchema corresponds to the JSON schema field "defInSameSchema".
	DefInSameSchema *Thing `json:"defInSameSchema,omitempty,omitzero" yaml:"defInSameSchema,omitempty" mapstructure:"defInSameSchema,omitempty"`
}

type Thing struct {
	// S corresponds to the JSON schema field "s".
	S *string `json:"s,omitempty,omitzero" yaml:"s,omitempty" mapstructure:"s,omitempty"`
}
