// Code generated by github.com/atombender/go-jsonschema, DO NOT EDIT.

package test

type BooleanAsSchema struct {
	// Id corresponds to the JSON schema field "id".
	Id *string `json:"id,omitempty,omitzero" yaml:"id,omitempty" mapstructure:"id,omitempty"`
}
