// Code generated by github.com/atombender/go-jsonschema, DO NOT EDIT.

package test

type Name string

type RefExternalFileNestedRefs struct {
	// MyExternalThing corresponds to the JSON schema field "myExternalThing".
	MyExternalThing *Thing `json:"myExternalThing,omitempty,omitzero" yaml:"myExternalThing,omitempty" mapstructure:"myExternalThing,omitempty"`
}

type RefNested struct {
	// MyThing corresponds to the JSON schema field "myThing".
	MyThing *Thing `json:"myThing,omitempty,omitzero" yaml:"myThing,omitempty" mapstructure:"myThing,omitempty"`
}

type Thing struct {
	// Name corresponds to the JSON schema field "name".
	Name *Name `json:"name,omitempty,omitzero" yaml:"name,omitempty" mapstructure:"name,omitempty"`
}
